%	6 July 2021
%	simulation for h estimation
%	Berenguer-Rico, Johansen, Nielsen (2021)
%	A model where the Least Trimmed Squares estimator is maximum likelihood

%	User should change
%	Sample size in line 21
%	DGP in lines 58,59


tic
clear;
clc;
rng(100);
format short;

% This program runs Monte Carlo experiments for the
% estimation of h via the Thiele statistic

rep=1000;                            % Number of Replicas
n=400;                               % Sample size: we consider n=25, 100, 400, 1600, 12800

% DGP Parameters
gamma=0.8;                           % Proportion of good observations
h=floor(n*gamma);                    % Number of good observation in the DGP
nout=n-h;                            % Number of outliers in the DGP
beta=0;                              % Regression coefficient: location

% Estimation Parameters
minr=0.6;
maxr=0.9;
hrange=(floor(minr*n):floor(maxr*n))';
shrange=size(hrange);
rhrange=shrange(1,1);

% Matrices to collect results
kum4=zeros(rhrange,1);
kum3=zeros(rhrange,1);
sig2LTShjkeep=zeros(rhrange,1);
kum4hj=zeros(rhrange,1);
hThiele=zeros(rep,1);

% Monte Carlo starts

v=1;
while v <= rep

% Data Generating Process

    % Good Errors

        ei=randn(h,1);                                       % Normal: DGPs 1,2
        

    % Outliers:

        %  G distribution = normal + shift in middle
        % eoutbs=randn(nout,1);  shiftp=0;   shiftn=-0;      % Normal:  DGP 1 
        eoutbs=randn(nout,1);  shiftp=3;   shiftn=-1;      % Normal:  DGP 2

        idxbs=(eoutbs>=0);
        eoutr=shiftp+eoutbs(idxbs);   
        eoutl=shiftn+eoutbs(~idxbs);

        % Combine Good and Outlier distributions
        ejr=max(ei)+eoutr;                % to the right
        ejl=min(ei)+eoutl;                % to the left

        % All errors (sorted)
        eis=sort(ei);
        ejrs=sort(ejr);
        ejls=sort(ejl);
        eall=vertcat(ejls, eis, ejrs);
    
% Location-Scale Model
yis=beta+eall;

% LTS estimation 

cs1=cumsum(yis);
cs10=vertcat(0,cs1);
cs2=cumsum(yis.^2);
cs20=vertcat(0,cs2);
cs3=cumsum(yis.^3);
cs30=vertcat(0,cs3);
cs4=cumsum(yis.^4);
cs40=vertcat(0,cs4);

% LTS estimation for each h_j (hestim)

j=1;
while j<= rhrange
    
    hestim=hrange(j);
    % This is are vectors of (n-hestim+1) x 1 variances
    sig2LTSall=(1/hestim)*(cs20(hestim+1:n+1)-cs20(1:n-hestim+1))-((1/hestim)*(cs10(hestim+1:n+1)-cs10(1:n-hestim+1))).^2;
    mu4LTSall=(1/hestim)*(cs40(hestim+1:n+1)-cs40(1:n-hestim+1))...
        -4.*((1/hestim)*(cs30(hestim+1:n+1)-cs30(1:n-hestim+1))).*(((1/hestim)*(cs10(hestim+1:n+1)-cs10(1:n-hestim+1))))...
        +6.*((1/hestim)*(cs20(hestim+1:n+1)-cs20(1:n-hestim+1))).*(((1/hestim)*(cs10(hestim+1:n+1)-cs10(1:n-hestim+1))).^2)...
        -3.*(((1/hestim)*(cs10(hestim+1:n+1)-cs10(1:n-hestim+1))).^4);
    mu3LTSall=(1/hestim)*(cs30(hestim+1:n+1)-cs30(1:n-hestim+1))...
        -3.*((1/hestim)*(cs20(hestim+1:n+1)-cs20(1:n-hestim+1))).*((1/hestim)*(cs10(hestim+1:n+1)-cs10(1:n-hestim+1)))...
        +2.*(((1/hestim)*(cs10(hestim+1:n+1)-cs10(1:n-hestim+1))).^3);
    
   
      % LTS estimator of sig2 and its corresponding 4th moment
      [sig2LTShj,psig2LTShj]=min(sig2LTSall);
      %sig2LTShjkeep(j)=sig2LTShj;
      mu4LTShj=mu4LTSall(psig2LTShj,1);
      mu3LTShj=mu3LTSall(psig2LTShj,1);
      
        
     % 4th cumulant for each hj
     kum4(j)=hestim*(       (   (mu4LTShj/((sig2LTShj).^2))  -3   ).^2     )/24; 
     % 3rd cumulant for each hj
     kum3(j)=hestim*(       (   (mu3LTShj/((sig2LTShj).^(3/2)))  -0   ).^2     )/6; 
     % Thiele statistic
         
j=j+1;
end


% h estimation for LTS

Thiele=kum4+kum3;

[minThiele,pThiele]=min(Thiele);
hThiele(v)=hrange(pThiele,1);

v=v+1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

m_trueh_M=[min(hrange) h max(hrange)]
m_q5_mean_meadian_q95_M=[min(hThiele) mean(hThiele) max(hThiele)]
bias_hkum4LTS=mean(hThiele)-h
sd_hkum4LTS=std(hThiele) 
hist(hThiele)
timeElapsed = toc