%	6 July 2021
%	simulation of distribution of t-statistics: DGPs 1-3
%	Berenguer-Rico, Johansen, Nielsen (2021)
%	A model where the Least Trimmed Squares estimator is maximum likelihood

%	User should change
%	Sample size in line 22
%	DGP in lines 54-55 and in 60-64

tic
clear;
clc;
rng(100);
format short;

% This program runs Monte Carlo experiments for the
% OLS, LTS, LMS and SLTS estimators in the location-scale model.
% Inferences on the location parameter in DGPs: 1,2,3 (in Table 7.1)
% LTS and LMS models

rep=1000000;                                 % Number of Replicas
n=1600;                                      % Sample size: we consider n = 25, 100, 400, 1600
gamma=0.8;                                   % Proportion of good observations
h=n*gamma;                                   % Number of good observation in the DGP
nout=n-h;                                    % Number of outliers in the DGP
gammaestim=gamma;                            % Proportion of good observations selected by LTS and LMS estimators
hestim=n*gammaestim;                         % Number of good observations selected by LTS and LMS estimators
qNormal=norminv((1+gammaestim)/2);           % Normal Quantile at (1+gammaestim)/2
fNormal=normpdf(qNormal);                    % Normal density at qNormal 
cf2LTS=(1-2*qNormal*fNormal/gammaestim)^2;   % Consistency factor LTS squared
beta=0;                                      % Regression coefficient: location

% Matrices to collect results
betaOLS=zeros(rep,1);
sigmauOLS=zeros(rep,1);
tbetaOLS=zeros(rep,1);
betaLTS=zeros(rep,1);
sigmauLTS=zeros(rep,1);
tbetaLTS=zeros(rep,1);
betaLMS=zeros(rep,1);
sigmauLMS=zeros(rep,1);
tbetaLMS=zeros(rep,1);
tbetaSLTS=zeros(rep,1);

% Monte Carlo starts

v=1;
while v <= rep

% Data Generating Process

% Good Errors

    ei=randn(h,1);                       % Normal          DGP 1,2
    ei=-1 + (1+1)*rand(h,1);               % Uniform[-1,1]   DGP 3

% Outliers:

    %  G distribution = normal + shift in middle
    % eoutbs=randn(nout,1);  shiftp=0;   shiftn=0;                 % Normal: DGP 1 
    % eoutbs=randn(nout,1);  shiftp=3;   shiftn=-1;              % Normal: DGP 2 

    %  G distribution = uniform + shift in middle
    eoutbs=-1 + (1+1)*rand(nout,1);   shiftp=3;   shiftn=-1;   % Uniform: DGP 3
   
    
    idxbs=(eoutbs>=0);
    eoutr=shiftp+eoutbs(idxbs);   
    eoutl=shiftn+eoutbs(~idxbs);

    % Outliers Conditional on the extreme values of the good
    ejr=max(ei)+eoutr;                % to the right
    ejl=min(ei)+eoutl;                % to the left

    % All errors (sorted)
    eis=sort(ei);
    ejrs=sort(ejr);
    ejls=sort(ejl);
    eall=vertcat(ejls, eis, ejrs);
    
% Location-Scale Model
yis=beta+eall;

% OLS Estimation
s1=mean(yis);
s2=mean(yis.^2);

betaOLS(v)=s1;
varuOLS=s2-(s1.^2);
sigmauOLS(v)=sqrt(varuOLS);
tbetaOLS(v)=(betaOLS(v)-beta)./sqrt(varuOLS/n);


% LTS estimation 

cs1=cumsum(yis);
cs10=vertcat(0,cs1);
cs2=cumsum(yis.^2);
cs20=vertcat(0,cs2);

% This is a vector of (n-hestim+1) x 1 variances
muLTSall=(1/hestim)*(cs10(hestim+1:n+1)-cs10(1:n-hestim+1));
sig2LTSall=(1/hestim)*(cs20(hestim+1:n+1)-cs20(1:n-hestim+1))-((1/hestim)*(cs10(hestim+1:n+1)-cs10(1:n-hestim+1))).^2;

[minsig2LTS,psig2LTS]=min(sig2LTSall);
sigmauLTS(v)=sqrt(minsig2LTS);
betaLTS(v)=muLTSall(psig2LTS,1); 
tbetaLTS(v)=(betaLTS(v)-beta)./sqrt(minsig2LTS/hestim);    


% LMS estimation
muLMSall=(yis(hestim:n)+yis(1:n-hestim+1))./2;
sigLMSall=(yis(hestim:n)-yis(1:n-hestim+1))./2;
[minsigLMS,psigLMS]=min(sigLMSall);
betaLMS(v)=muLMSall(psigLMS,1);
sigmauLMS(v)=minsigLMS;
tbetaLMS(v)=hestim*(betaLMS(v)-beta)./minsigLMS;


% RLTS t-statistic: correction factors
tbetaSLTS(v)=sqrt(hestim)*(betaLTS(v)-beta)./sqrt(minsig2LTS/cf2LTS);

v=v+1;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%55

% Size & Power

counttbetaOLS=0;
counttbetaLTS=0;
counttbetaLMS=0;
counttbetaSLTS=0;

i=1;
while i <= rep

if abs(tbetaOLS(i)) > 1.96    
counttbetaOLS=counttbetaOLS+1;
end    
    
if abs(tbetaLTS(i)) > 1.96    
counttbetaLTS=counttbetaLTS+1;
end

if abs(tbetaLMS(i)) > -1*sign(0.975-0.5)*log(1-(2*abs(0.975-0.5)))
counttbetaLMS=counttbetaLMS+1;
end

if abs(tbetaSLTS(i)) > 1.96    
counttbetaSLTS=counttbetaSLTS+1;
end


i=i+1;
end

'Size / Power: CUSQ';
prtbetaOLS=counttbetaOLS/rep;
prtbetaLTS=counttbetaLTS/rep;
prtbetaLMS=counttbetaLMS/rep;
prtbetaSLTS=counttbetaSLTS/rep;

[prtbetaOLS prtbetaLTS prtbetaLMS prtbetaSLTS]

timeElapsed = toc